///////////////////////////////////////////////////////
//													 //
//           Ray Tracer 1.0 by Ryan Liu              //
//			     www.ryan-liu.com					 //
//													 //
///////////////////////////////////////////////////////
/**
*   Tidy up to AS3 by FoxLively 2008.11
*/
package
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.filters.BlurFilter;
	import flash.filters.GlowFilter;
	
	public class RayTracer extends Sprite
	{
		/**
		 *循环次数 默认为1  为-1时 无限循环 
		 */		
		private var loop:int = 1;
		/**
		 *光线的宽度 
		 */		
		public var rayWidth:uint = 10;
		/**
		 *光线的长度 
		 */		
		public var rayLength:uint = 16;
		/**
		 *光线的颜色
		 */		
		public var rayColor:uint = 0xFFFFFF;
		/**
		 *光线的透明度 
		 */		
		public var rayAlpha:Number = 1;
		/**
		 *光线的外发光滤镜
		 */		
		public var glowFilter:GlowFilter = new GlowFilter(0xff6699,100,20,20);
		/**
		 *光线的模糊滤镜 
		 */		
		public var blurFilter:BlurFilter = new BlurFilter(8,8);
		
		private var trackArray:Array;
		private var _painter:Sprite  = new Sprite();;
		private var _dataArr0:Array = new Array();
		private var _dataArr1:Array = new Array();
		private var _dataArr2:Array = new Array();
		private var nowI:int = 0;
		public function RayTracer(){
			super();
			this.addChild(_painter);
		}
		/**
		 *根据一个没有角度ro的只有x,y坐标的数组  绘制光线，数组里每个元素都是一个有x,y坐标信息的Object对象，角度ro会通过x,y计算得到 
		 * @param _arr
		 * 
		 */		
		public function buildByNoRoArray(_arr:Array,_loop:int = 1):void{
			if(_arr.length<=1){//光线数据太少不进行绘制
				return ;
			}
			for(var i:int = 0;i<_arr.length-1;i++){
				_arr[i].ro = MathIn.angle(_arr[i+1].x-_arr[i].x,_arr[i+1].y-_arr[i].y);
			}
			_arr[_arr.length-1].ro = _arr[_arr.length-2].ro;
			this.buildByRoArray(_arr,_loop);
		}
		/**
		 *根据 一个 ，数组里每个元素都是一个Object对象 并有 x,y,ro属性
		 * @param _traceArray
		 * 
		 */		
		public function buildByRoArray(_traceArray:Array,_loop:int = 1):void{
			this.loop = _loop;
			_painter.filters = [this.glowFilter,this.blurFilter];
			this.trackArray = _traceArray;
			this.addEventListener(Event.ENTER_FRAME,onF);
		}
		private function onF(_e:Event):void{
			run();
		}
		//traceRayn 函数：绘制光线
		private function run():void {
			_painter.graphics.clear();
			//位置加进去
			if(nowI<this.trackArray.length){
				_dataArr0.push({x:this.trackArray[nowI].x, y:this.trackArray[nowI].y, ro:this.trackArray[nowI].ro+90});
				nowI++;
			}else{
				_dataArr0.shift();
			}
			if (_dataArr0.length>rayLength) {
				_dataArr0.shift();
			}
			if(_dataArr0.length==0){
				this.clear();
				return;
			}
			_dataArr1 = new Array();
			_dataArr2 = new Array();
			for (var i:int = 0; i<_dataArr0.length; i++) {
				var twidth:Number = rayWidth*MathIn.sin(180*i/_dataArr0.length);//定义曲线宽度   首尾  的 sin0 = sin180 = 0   中间 宽
				var tobj:Object = _dataArr0[i];
				var x1:Number = tobj.x+MathIn.cos(tobj.ro)*twidth/2;
				var y1:Number = tobj.y+MathIn.sin(tobj.ro)*twidth/2;
				var x2:Number = tobj.x-MathIn.cos(tobj.ro)*twidth/2;
				var y2:Number = tobj.y-MathIn.sin(tobj.ro)*twidth/2;
				_dataArr1.push({x:x1, y:y1});
				_dataArr2.push({x:x2, y:y2});
			}
			var alen:Number = _dataArr1.length;
			var cc:uint = rayColor;
			//绘制该线的外边框 以便在其上加边框
			_painter.graphics.beginFill(cc,rayAlpha);
//			_painter.graphics.lineStyle(1,0x0000FF);
			_painter.graphics.moveTo(_dataArr1[0].x,_dataArr1[0].y);
			for (i = 1; i<alen-1; i++) {
				_painter.graphics.lineTo(_dataArr1[i].x,_dataArr1[i].y);
			}
			for (i = alen-1; i>0; i--) {
				_painter.graphics.lineTo(_dataArr2[i].x,_dataArr2[i].y);
			}
			_painter.graphics.lineTo(_dataArr1[0].x,_dataArr1[0].y);
			_painter.graphics.endFill();
		}
		/**
		 * 清除光线 并停止绘制
		 * 
		 */		
		public function clear():void {
			this.nowI = 0;
			this.loop-=1;
			if(this.loop==0){
				trackArray = [];
				_painter.graphics.clear();
				this.removeEventListener(Event.ENTER_FRAME,onF);
			}
		}
	}
}
import flash.geom.Point;
	
	class MathIn
	{
		public static function sin(_ro:Number):Number{
			return Math.sin(MathIn.ro2pi(_ro));
		}
		public static function cos(_ro:Number):Number{
			return Math.cos(MathIn.ro2pi(_ro));
		}
		public static function atan(_v:Number):Number{
			return Math.atan(_v)*180/Math.PI;
		}
		public static function atan2(_y:Number,_x:Number):Number{
			return atan(_y/_x);
		}
		public static function ro2pi(_ro:Number):Number {
			return Math.PI/180*_ro;
		}
		public static function pi2ro(_pi:Number):Number {
			return _pi/Math.PI*180;
		}
		/**
		 *real angle/rotation value in flash coordinate space 
		 * @param _x
		 * @param _y
		 * @return 
		 * 
		 */		
		public static function angle(_x:Number, _y:Number):Number {
			if (_x == 0) {
				if (_y == 0) {
					return 0;
				} else if (_y>0) {
					return 90;
				} else if (_y<0) {
					return 270;
				}
			} else if (_x>0) {
				if (_y == 0) {
					return 0;
				} else if (_y>0) {
					return atan2(_y,_x);
				} else if (_y<0) {
					return 360-atan(-1*_y/_x);
				}
			}
			 else if (_x<0) {
				if (_y == 0) {
					return 180;
				} else if (_y>0) {
					return 180-atan(-1*_y/_x);
				} else if (_y<0) {
					return 180+atan2(_y,_x);
				}
			}
			return 0;
		}
	}